/*
 Copyright (c) 2009 Devin DeLong
 
 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the "Software"), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:
 
 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 */

#pragma once
#ifndef SHOTTYPEMANAGER_H
#define SHOTTYPEMANAGER_H

// STL
#include <map>
#include <string>

// bzfs
#ifndef _BZFS_API_H_
#include "bzfsAPI.h"
#endif

class ShotTypeManager
{
    
public:
  
  typedef std::map<bz_eTeamType, bz_eShotType>::iterator iterator;
  typedef std::map<bz_eTeamType, bz_eShotType>::const_iterator const_iterator;
  
  /** Resets the shot type for a team. */
  void reset(bz_eTeamType team) { set(team, eStandardShot); }
  
  /** Resets all shot types to the default types. */
  void reset()
  {
    for (iterator begin = this->begin(); begin != this->end(); ++begin)
      set(begin->first, eStandardShot);
  }
  
  /** Sets the shot type for a team. */
  void set(bz_eTeamType team, bz_eShotType shotType)
  {
    teamShotTypes[team] = shotType;
  }
  
  bool empty() const
  {
    return teamShotTypes.empty();
  }
  
  bool size() const
  {
    return teamShotTypes.size();
  }
  
  /** Gets the shot type assigned to a team. */
  bz_eShotType get(bz_eTeamType team) const 
  {
    const_iterator type = teamShotTypes.find(team);
    return type == this->end() ? eStandardShot : type->second;
  };
  
  iterator begin() { return teamShotTypes.begin(); }
  const_iterator begin() const { return teamShotTypes.begin(); }
  
  iterator end() { return teamShotTypes.end(); }
  const_iterator end() const { return teamShotTypes.end(); }
  
private:
  
  std::map<bz_eTeamType, bz_eShotType> teamShotTypes;
};


#endif /* SHOTTYPEMANAGER_H */
