/*
 Copyright (c) 2009 Devin DeLong
 
 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the "Software"), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:
 
 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 */

#pragma once
#ifndef BZU_LOCAL_PTR_HPP
#define BZU_LOCAL_PTR_HPP

#include "bz_delete.hpp"
#include "noncopyable.hpp"

namespace bzu {
  
  template <typename T> 
  class local_ptr : public noncopyable {
    
    class boolean_converter_type {
      void operator delete(void*); // Do not allow operator delete
    };
    
  public:
    
    typedef T element_type;
    typedef T* pointer_type;
    typedef T& reference_type;
    
    explicit local_ptr(pointer_type pointee = NULL) : _pointee(pointee) 
    {
      
    }
    
    ~local_ptr() 
    { 
      bz_delete(this->_pointee); 
    }
    
    reference_type operator*() const 
    { 
      return *this->_pointee; 
    }
    
    pointer_type operator->() const 
    { 
      return this->_pointee; 
    }
    
    // Conversion to bool
    operator boolean_converter_type*() const
    {
      static boolean_converter_type _bool_converter_address;
      return (this->_pointee == NULL) ? NULL : &_bool_converter_address;
    }
    
    bool operator!() const
    {
      return (this->_pointee == NULL);
    }
    
    void reset(pointer_type pointee = NULL)
    {
      local_ptr<T>(pointee).swap(*this); 
    }
    
    pointer_type get() const
    {
      return this->_pointee;
    }
    
    void swap(local_ptr<T>& ptr)
    {
      pointer_type temp = this->_pointee;
      this->_pointee = ptr._pointee;
      ptr._pointee = temp;
    }
    
  private:
    
    pointer_type _pointee;
  };
  
} // namespace bzf


#endif /* BZU_LOCAL_PTR_HPP */

