/*
 Copyright (c) 2009 Devin DeLong
 
 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the "Software"), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:
 
 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 */

#pragma once
#ifndef COMMAND_H
#define COMMAND_H

// BZU
#include "bzu/noncopyable.hpp"
#include "bzu/bz_types.hpp"
#include "bzu/messaging.hpp"

// STL
#include <string>
#include <vector>


#define TW_PRE_MESSAGE "[TeamWeapons->] "
#define TW_PRE_ERR_MESSAGE "[TeamWeapons.error->] "

class Command : public bzu::noncopyable
{    
  // Virtual implementation.
  virtual bool doCommand(const std::vector<std::string> & params) = 0;
  
  // Cloneable implementation.
  virtual Command * doClone() const = 0;
  
public:
  
  Command() : _player(BZ_SERVER) { }
  
  // Get the player who last called doCommand();
  bzu::PlayerID player() const { return _player; }
  
  // Cloneable interface.
  Command * clone() const
  {
    return doClone();
  }
  
  // Virtual interface
  bool operator()(bzu::PlayerID player, const std::vector<std::string> & params)
  {
    _player = player;
    return doCommand(params);
  }
  
  virtual ~Command() {  }
  
private:
  
  bzu::PlayerID _player;  
};


inline bool sendMessageTW(bzu::PlayerID player, const std::string& msg)
{
  return bzu::sendMessage(player, std::string(TW_PRE_MESSAGE) + msg);
}

inline bool sendErrMessageTW(bzu::PlayerID player, const std::string& msg)
{
  return bzu::sendMessage(player, std::string(TW_PRE_ERR_MESSAGE) + msg);
}

#undef TW_PRE_MESSAGE
#undef TW_PRE_ERR_MESSAGE

#endif /* COMMAND_H */
