/*
 Copyright (c) 2009 Devin DeLong
 
 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the "Software"), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:
 
 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 */

// Plugin
#include "CommandHandler.h"
#include "commands.h"

// bzu
#include "bzu/bz_types.hpp"

// STL
#include <vector>
#include <map>
#include <string>

// Command defines
#define SET_COMMAND "set"
#define INFO_COMMAND "info"


CommandHandler::CommandHandler() 
{ 
  // Lazy initialization would be good here, but for now,
  // lets just allocate the commands.
  commandMap[SET_COMMAND] = new SetCommand();
  commandMap[INFO_COMMAND] = new InfoCommand();
}


// Clean up memory if we need to.
CommandHandler::~CommandHandler()
{
  for (CommandMap::iterator begin = commandMap.begin(); begin != commandMap.end(); ++begin)
    if (begin->second)
      delete begin->second;
}


// Perform the command.
bool CommandHandler::handle(bzu::PlayerID player, 
                            bzu::String /*cmd*/, 
                            bzu::String /*msg*/, 
                            bzu::StringList* cmdParams)
{
  // Do nothing if no actions are passed.
  // TODO: maybe print usage info.
  if (!cmdParams->size())
    return true;
  
  // Get the action
  std::string actionStr = cmdParams->get(0).c_str();  
  
  // Move action params into an std::vector.
  std::vector<std::string> params;
  for (std::size_t i = 1; i < cmdParams->size(); ++i)
    params.push_back(cmdParams->get(i).c_str());
  
  CommandMap::iterator action = commandMap.find(actionStr);
  
  // Make sure the command exists
  if (action == commandMap.end())
    return false;
  
  // Call the action
  return action->second->operator()(player, params);
}







