/*
 Copyright (c) 2009 Devin DeLong
 
 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the "Software"), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:
 
 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 */

// bzu
#include "bzu/FlagInfo.h"

// STL
#include <string>


// Used to optimize searching for the static functions

#define N_NONSTANDARD_SHOT_FLAGS 11
#define N_BAD_FLAGS 14
#define N_TEAM_FLAGS 4
#define N_STANDARD_SHOT_FLAGS TOTAL_FLAGS - N_NONSTANDARD_SHOT_FLAGS
#define N_SUPER_FLAGS TOTAL_FLAGS - (N_TEAM_FLAGS + N_BAD_FLAGS)

#define FLAG_BEGIN 0
#define FLAG_END TOTAL_FLAGS

#define SUPERFLAG_BEGIN FLAG_BEGIN
#define SUPERFLAG_END N_SUPER_FLAGS

#define GOODFLAG_BEGIN FLAG_BEGIN
#define GOODFLAG_END N_SUPER_FLAGS + N_TEAM_FLAGS

#define NONSTANDARDSHOT_BEGIN FLAG_BEGIN
#define NONSTANDARDSHOT_END N_NONSTANDARD_SHOT_FLAGS

#define STANDARDSHOT_BEGIN NONSTANDARDSHOT_END
#define STANDARDSHOT_END FLAG_END

#define TEAMFLAG_BEGIN SUPERFLAG_END
#define TEAMFLAG_END N_SUPER_FLAGS + N_TEAM_FLAGS

#define BADFLAG_BEGIN TEAMFLAG_END
#define BADFLAG_END FLAG_END

namespace bzu
{
// FlagInfo::FlagInfoData data
//------------------------------------------------------------------------------//

const FlagInfo::FlagInfoData FlagInfo::_validFlags[TOTAL_FLAGS] = {
  
  // Super flags
  FlagInfo::FlagInfoData(GUIDED_MISSILE, eGMShot, eGoodFlag, eNoTeam, "Guided Missile"),
  FlagInfo::FlagInfoData(LASER, eLaserShot, eGoodFlag, eNoTeam, "Laser"),
  FlagInfo::FlagInfoData(THIEF, eThiefShot, eGoodFlag, eNoTeam, "Thief"),
  FlagInfo::FlagInfoData(SUPPER_BULLET, eSuperShot, eGoodFlag, eNoTeam, "Super Bullet"),
  FlagInfo::FlagInfoData(PHANTOM_ZONE, ePhantomShot, eGoodFlag, eNoTeam, "Phantom Zone"),
  FlagInfo::FlagInfoData(SHOCKWAVE, eShockWaveShot, eGoodFlag, eNoTeam, "ShockWave"),
  FlagInfo::FlagInfoData(RICOCHET, eRicoShot, eGoodFlag, eNoTeam, "Ricochet"),
  FlagInfo::FlagInfoData(MACHINE_GUN, eMachineGunShot, eGoodFlag, eNoTeam, "Machine Gun"),
  FlagInfo::FlagInfoData(INVISIBLE_BULLET, eInvisibleShot, eGoodFlag, eNoTeam, "Invisible Bullet"),
  FlagInfo::FlagInfoData(CLOAKED_SHOT, eCloakedShot, eGoodFlag, eNoTeam, "Cloaked Shot"),
  FlagInfo::FlagInfoData(RAPID_FIRE, eRapidFireShot, eGoodFlag, eNoTeam, "Rapid Fire"),
  
  // Super flags with standard shots
  FlagInfo::FlagInfoData(GENOCIDE, eStandardShot, eGoodFlag, eNoTeam, "Genocide"),
  FlagInfo::FlagInfoData(AGILITY, eStandardShot, eGoodFlag, eNoTeam, "Agility"), 
  FlagInfo::FlagInfoData(BURROW, eStandardShot, eGoodFlag, eNoTeam, "Burrow"), 
  FlagInfo::FlagInfoData(CLOAKING, eStandardShot, eGoodFlag, eNoTeam, "Cloak"), 
  FlagInfo::FlagInfoData(IDENTIFY, eStandardShot, eGoodFlag, eNoTeam, "Identify"), 
  FlagInfo::FlagInfoData(JUMPING, eStandardShot, eGoodFlag, eNoTeam, "Jumping"), 
  FlagInfo::FlagInfoData(LOW_GRAVITY, eStandardShot, eGoodFlag, eNoTeam, "Low Gravity"),
  FlagInfo::FlagInfoData(MASQUERADE, eStandardShot, eGoodFlag, eNoTeam, "Masquerade"),
  FlagInfo::FlagInfoData(NARROW, eStandardShot, eGoodFlag, eNoTeam, "Narrow"),
  FlagInfo::FlagInfoData(OSCILLATION_OVERTHRUSTER, eStandardShot, eGoodFlag, eNoTeam, 
               "Oscillation Overthruster"),
  FlagInfo::FlagInfoData(QUICK_TURN, eStandardShot, eGoodFlag, eNoTeam, "Quick Turn"),
  FlagInfo::FlagInfoData(SEER, eStandardShot, eGoodFlag, eNoTeam, "Seer"),
  FlagInfo::FlagInfoData(SHIELD, eStandardShot, eGoodFlag, eNoTeam, "Shield"),
  FlagInfo::FlagInfoData(STEALTH, eStandardShot, eGoodFlag, eNoTeam, "Stealth"),
  FlagInfo::FlagInfoData(STEAM_ROLLER, eStandardShot, eGoodFlag, eNoTeam, "Steam Roller"),
  FlagInfo::FlagInfoData(TINY, eStandardShot, eGoodFlag, eNoTeam, "Tiny"),
  FlagInfo::FlagInfoData(USELESS, eStandardShot, eGoodFlag, eNoTeam, "Useless"),
  FlagInfo::FlagInfoData(HIGH_SPEED, eStandardShot, eGoodFlag, eNoTeam, "High Speed"),
  FlagInfo::FlagInfoData(WINGS, eStandardShot, eGoodFlag, eNoTeam, "Wings"),
  
  // Team flags
  FlagInfo::FlagInfoData(RED_TEAM, eStandardShot, eGoodFlag, eRedTeam, "Red Team"),
  FlagInfo::FlagInfoData(GREEN_TEAM, eStandardShot, eGoodFlag, eGreenTeam, "Green Team"),
  FlagInfo::FlagInfoData(BLUE_TEAM, eStandardShot, eGoodFlag, eBlueTeam, "Blue Team"),
  FlagInfo::FlagInfoData(PURPLE_TEAM, eStandardShot, eGoodFlag, ePurpleTeam, "Purple Team"),
  
  // Bad flag
  FlagInfo::FlagInfoData(BLINDNESS, eStandardShot, eBadFlag, eNoTeam, "Blindness"),
  FlagInfo::FlagInfoData(BOUNCY, eStandardShot, eBadFlag, eNoTeam, "Bouncy"),
  FlagInfo::FlagInfoData(COLORBLINDNESS, eStandardShot, eBadFlag, eNoTeam, "Color Blindness"),
  FlagInfo::FlagInfoData(FORWARD_ONLY, eStandardShot, eBadFlag, eNoTeam, "Forward Only"),
  FlagInfo::FlagInfoData(JAMMING, eStandardShot, eBadFlag, eNoTeam, "Jamming"),
  FlagInfo::FlagInfoData(LEFT_TURN_ONLY, eStandardShot, eBadFlag, eNoTeam, "Left Turn Only"),
  FlagInfo::FlagInfoData(MOMENTUM, eStandardShot, eBadFlag, eNoTeam, "Momentum"),
  FlagInfo::FlagInfoData(NO_JUMPING, eStandardShot, eBadFlag, eNoTeam, "No Jumping"),
  FlagInfo::FlagInfoData(OBESITY, eStandardShot, eBadFlag, eNoTeam, "Obese"), 
  FlagInfo::FlagInfoData(REVERSE_CONTROLS, eStandardShot, eBadFlag, eNoTeam, "Reverse Controls"), 
  FlagInfo::FlagInfoData(REVERSE_ONLY, eStandardShot, eBadFlag, eNoTeam, "Reverse Only"), 
  FlagInfo::FlagInfoData(RIGHT_TURN_ONLY, eStandardShot, eBadFlag, eNoTeam, "Right Turn Only"), 
  FlagInfo::FlagInfoData(TRIGGERHAPPY, eStandardShot, eBadFlag, eNoTeam, "Trigger Happy"), 
  FlagInfo::FlagInfoData(WIDE_ANGLE, eStandardShot, eBadFlag, eNoTeam, "Wide Angle") 
};


// FlagInfo functions
//------------------------------------------------------------------------------//

FlagInfo::FlagInfo(const std::string& flag) 
{
  set(flag);
}


FlagInfo::FlagInfo(int ID)
{
  set(ID);
}


void FlagInfo::set(const std::string& flag)
{
  std::size_t index = find(flag);
  
  if(index != FLAG_END) {
    _valid = true;
    _index = index;
  } else {
    _valid = false;
    _index = 0;
  }
}


void FlagInfo::set(int ID)
{
  std::size_t index = find(ID);
  
  if(index != FLAG_END) {
    _valid = true;
    _index = index;
  } else {
    _valid = false;
    _index = 0;
  }
}


// Find functions
//------------------------------------------------------------------------------//

std::size_t FlagInfo::find(const std::string& flag) const
{  
  for(std::size_t index = FLAG_BEGIN; index < FLAG_END; ++index) {
    if(flag == _validFlags[index].getName())
      return index;
  }
  
  return FLAG_END;
}


// Searches [begin, end)
std::size_t FlagInfo::find(const std::string& flag, std::size_t begin, std::size_t end)
{
  for(std::size_t index = begin; index < end; ++index) {
    if(flag == _validFlags[index].getName())
      return index;
  }
  
  return FLAG_END;
}


std::size_t FlagInfo::find(int ID) const
{
  std::string flag = bz_getFlagName(ID).c_str();
  
  // Check if ID is valid
  if(flag == "")
    return FLAG_END;
    
  for(std::size_t index = FLAG_BEGIN; index < FLAG_END; ++index) {
    if(flag == _validFlags[index].getName())
      return index;
  }
  
  return FLAG_END;
}


// Simple utility functions
//------------------------------------------------------------------------------//

bool FlagInfo::isGood(const std::string& flag)
{
  std::size_t index = find(flag, GOODFLAG_BEGIN, GOODFLAG_END);
  
  if(index == FLAG_END)
    return false;
  
  return true;
}


bool FlagInfo::isBad(const std::string& flag)
{
  std::size_t index = find(flag, BADFLAG_BEGIN, BADFLAG_END);
  
  if(index == FLAG_END)
    return false;
  
  return true;
}


bool FlagInfo::isTeam(const std::string& flag)
{
  std::size_t index = find(flag, TEAMFLAG_BEGIN, TEAMFLAG_END);
  
  if(index == FLAG_END)
    return false;
  
  return true;
}


bool FlagInfo::isSuper(const std::string& flag)
{
  std::size_t index = find(flag, SUPERFLAG_BEGIN, SUPERFLAG_END);
  
  if(index == FLAG_END)
    return false;
  
  return true;
}


bool FlagInfo::hasStandardShot(const std::string& flag)
{
  std::size_t index = find(flag, STANDARDSHOT_BEGIN, STANDARDSHOT_END);
  
  if(index == FLAG_END)
    return false;
  
  return true;
}


bool FlagInfo::hasNonStandardShot(const std::string& flag)
{
  std::size_t index = find(flag, NONSTANDARDSHOT_BEGIN, NONSTANDARDSHOT_END);
  
  if(index == FLAG_END)
    return false;
  
  return true;
}


// Some more simple utility functions
//------------------------------------------------------------------------------//

bool FlagInfo::isGood() const
{
  if(!_valid)
    return false;
  
  if(_validFlags[_index].getQuality() == eGoodFlag)
    return true;    
  
  return false;
}

bool FlagInfo::isBad() const
{
  if(!_valid)
    return false;
  
  if(_validFlags[_index].getQuality() == eBadFlag)
    return true;    
  
  return false;
}

bool FlagInfo::isTeam() const
{
  if(!_valid)
    return false;
  
  if(_validFlags[_index].getTeam() == eNoTeam)
    return false;    
  
  return true;
}

bool FlagInfo::isSuper() const
{
  if(!_valid)
    return false;
  
  if(_validFlags[_index].getTeam() == eNoTeam &&
     _validFlags[_index].getQuality() == eGoodFlag)
    return true;    
  
  return false;
}

bool FlagInfo::hasStandardShot() const
{
  if(!_valid)
    return false;
  
  if(_validFlags[_index].getShotType() == eStandardShot)
    return true;    
  
  return false;
}

bool FlagInfo::hasNonStandardShot() const
{
  if(!_valid)
    return false;
  
  if(_validFlags[_index].getShotType() != eStandardShot)
    return true;    
  
  return false;
}

} // namespace bzu

// The End
//------------------------------------------------------------------------------//










