/*
 Copyright (c) 2009 Devin DeLong
 
 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the "Software"), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:
 
 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 */


// Plugin
#include "commands/InfoCommand.h"
#include "Global.h"

// bzu
#include "bzu/messaging.hpp"


// STL
#include <vector>
#include <map>
#include <string>

// bzfs
#ifndef _BZFS_API_H_
#include "bzfsAPI.h"
#endif

// Printable teams.
#define P_TEAM_RED "Red Team"
#define P_TEAM_GREEN "Green Team"
#define P_TEAM_BLUE "Blue Team"
#define P_TEAM_PURPLE "Purple Team"
#define P_TEAM_ROGUE "Rogue Team"
#define P_TEAM_HUNTER "Hunter Team"
#define P_TEAM_RABBIT "Rabbit Team"

// Making up for downfalls of FlagInfo
// That class needs to be rewritten.
// It could be so much better.
#define P_SHOT_STANDARD "Standard"
#define P_SHOT_SW "Shockwave"
#define P_SHOT_GM "Guided Missile"
#define P_SHOT_L "Laser"
#define P_SHOT_TH "Thief"
#define P_SHOT_SB "Super Bullet"
#define P_SHOT_PZ "Phantom Zone"
#define P_SHOT_R "Ricochet"
#define P_SHOT_MG "Machine Gun"
#define P_SHOT_IB "Invisible Bullet"
#define P_SHOT_CS "Cloaked Shot"
#define P_SHOT_RF "Rapid Fire"

bool InfoCommand::doCommand(const std::vector<std::string> & params)
{
  if (params.size())
    return false;
  
  if (Global::shotManager.empty())
  {
    sendMessageTW(player(), "No shot types set");
    return true;
  }
  
  for (ShotTypeManager::const_iterator begin = Global::shotManager.begin(); 
       begin != Global::shotManager.end(); ++begin)
  {
    std::string message = printableTeams[begin->first];
    message += " : ";
    message += printableShotTypes[begin->second];
    
    sendMessageTW(player(), message);
  } 
  
  return true;
}

InfoCommand::InfoCommand()
{
  printableTeams[eRedTeam] = P_TEAM_RED;
  printableTeams[eGreenTeam] = P_TEAM_GREEN;
  printableTeams[eBlueTeam] = P_TEAM_BLUE;
  printableTeams[ePurpleTeam] = P_TEAM_PURPLE;
  printableTeams[eRogueTeam] = P_TEAM_ROGUE;
  printableTeams[eHunterTeam] = P_TEAM_HUNTER;
  printableTeams[eRabbitTeam] = P_TEAM_RABBIT;
  
  printableShotTypes[eStandardShot] = P_SHOT_STANDARD;
  printableShotTypes[eShockWaveShot] = P_SHOT_SW;
  printableShotTypes[eGMShot] = P_SHOT_GM;
  printableShotTypes[eLaserShot] = P_SHOT_L;
  printableShotTypes[eThiefShot] = P_SHOT_TH;
  printableShotTypes[eSuperShot] = P_SHOT_SB;
  printableShotTypes[ePhantomShot] = P_SHOT_PZ;
  printableShotTypes[eRicoShot] = P_SHOT_R;
  printableShotTypes[eMachineGunShot] = P_SHOT_MG;
  printableShotTypes[eInvisibleShot] = P_SHOT_IB;
  printableShotTypes[eCloakedShot] = P_SHOT_CS;
  printableShotTypes[eRapidFireShot] = P_SHOT_RF;
}

