/*
 Copyright (c) 2009 Devin DeLong
 
 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the "Software"), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:
 
 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 */

// Plugin
#include "commands/SetCommand.h"
#include "Global.h"

// bzu
#include "bzu/FlagInfo.h"
#include "bzu/messaging.hpp"

// STL
#include <vector>
#include <string>

// bzfs
#ifndef _BZFS_API_H_
#include "bzfsAPI.h"
#endif

// Command param defines
#define RED_TEAM_C "red"
#define BLUE_TEAM_C "blue"
#define GREEN_TEAM_C "green"
#define PURPLE_TEAM_C "purple"
#define ROGUE_TEAM_C "rogue"
#define RABBIT_TEAM_C "rabbit"
#define HUNTER_TEAM_C "hunter"

#define STANDARD_TYPE_C "standard"

SetCommand::SetCommand()
{
  teamMap[RED_TEAM_C] = eRedTeam;
  teamMap[GREEN_TEAM_C] = eGreenTeam;
  teamMap[BLUE_TEAM_C] = eBlueTeam;
  teamMap[PURPLE_TEAM_C] = ePurpleTeam;
  teamMap[ROGUE_TEAM_C] = eRogueTeam;
  teamMap[RABBIT_TEAM_C] = eRabbitTeam;
  teamMap[HUNTER_TEAM_C] = eHunterTeam;
}


// Mostly just a bunch of messaging.
// It's good enough.
bool SetCommand::doCommand(const std::vector<std::string> & params)
{
  if (params.size() != 2)
    return false;
  
  const std::string& teamStr = params[0];
  TeamMap::const_iterator team = teamMap.find(teamStr);
  
  // Check the team
  if (team == teamMap.end())
  {
    sendErrMessageTW(player(), std::string("Invalid team : ") + teamStr);
    return true;
  }
  
  const std::string flagStr = params[1];
  
  // Check the flag/shot type
  if (flagStr == STANDARD_TYPE_C)
  {
    Global::shotManager.reset(team->second);
  } 
  else
  {    
    bzu::FlagInfo fInfo(flagStr);

    if (fInfo.isValid())
    {
      if (fInfo.hasNonStandardShot())
      {
        Global::shotManager.set(team->second, fInfo.getShotType());
      }
      else
      {
        sendErrMessageTW(player(), std::string("Invalid flag: ") + flagStr);
        sendErrMessageTW(player(), "Flags must have a non-standard shot type.");
        return true;
      }    
    }
    else
    {
      sendErrMessageTW(player(), std::string("Flag does not exist: ") + flagStr);
      return true;
    }    
  }
  
  // Send a confirmation message.
  // If we get here, everything is correct.
  std::string msg("Shot type set for ");
  msg += teamStr;
  msg += ": ";
  msg += flagStr;
  sendMessageTW(player(), msg);
  
  return true;
}

